@@ -851,7 +851,7 @@ void internal_dump_inode(FILE *out, const char *prefix,
 			inode->i_file_acl | ((long long)
 				(inode->osd2.linux2.l_i_file_acl_high) << 32),
 			LINUX_S_ISDIR(inode->i_mode) ? inode->i_dir_acl : 0);
-	if (os == EXT2_OS_LINUX)
+	if (os != EXT2_OS_HURD)
 		fprintf(out, "%sLinks: %d   Blockcount: %llu\n",
 			prefix, inode->i_links_count,
 			(((unsigned long long)
@@ -908,8 +908,7 @@ void internal_dump_inode(FILE *out, const char *prefix,
 		internal_dump_inode_extra(out, prefix, inode_num,
 					  (struct ext2_inode_large *) inode);
 	dump_inode_attributes(out, inode_num);
-	if (current_fs->super->s_creator_os == EXT2_OS_LINUX &&
-	    ext2fs_has_feature_metadata_csum(current_fs->super)) {
+	if (ext2fs_has_feature_metadata_csum(current_fs->super)) {
 		__u32 crc = inode->i_checksum_lo;
 		if (is_large_inode &&
 		    large_inode->i_extra_isize >=
