@@ -583,7 +583,7 @@ static errcode_t ext2fs_journal_load(journal_t *journal)
 	case JFS_DESCRIPTOR_BLOCK:
 	case JFS_COMMIT_BLOCK:
 	case JFS_REVOKE_BLOCK:
-		return EXT2_ET_CORRUPT_SUPERBLOCK;
+		return EXT2_ET_CORRUPT_JOURNAL_SB;
 
 	/* If we don't understand the superblock major type, but there
 	 * is a magic number, then it is likely to be a new format we
@@ -600,15 +600,15 @@ static errcode_t ext2fs_journal_load(journal_t *journal)
 
 	/* Checksum v1-3 are mutually exclusive features. */
 	if (jfs_has_feature_csum2(journal) && jfs_has_feature_csum3(journal))
-		return EXT2_ET_CORRUPT_SUPERBLOCK;
+		return EXT2_ET_CORRUPT_JOURNAL_SB;
 
 	if (journal_has_csum_v2or3(journal) &&
 	    jfs_has_feature_checksum(journal))
-		return EXT2_ET_CORRUPT_SUPERBLOCK;
+		return EXT2_ET_CORRUPT_JOURNAL_SB;
 
 	if (!ext2fs_journal_verify_csum_type(journal, jsb) ||
 	    !ext2fs_journal_sb_csum_verify(journal, jsb))
-		return EXT2_ET_CORRUPT_SUPERBLOCK;
+		return EXT2_ET_CORRUPT_JOURNAL_SB;
 
 	if (journal_has_csum_v2or3(journal))
 		journal->j_csum_seed = jbd2_chksum(journal, ~0, jsb->s_uuid,
@@ -618,12 +618,12 @@ static errcode_t ext2fs_journal_load(journal_t *journal)
 	 * format to be able to proceed safely, so any other checks that
 	 * fail we should attempt to recover from. */
 	if (jsb->s_blocksize != htonl(journal->j_blocksize))
-		return EXT2_ET_CORRUPT_SUPERBLOCK;
+		return EXT2_ET_CORRUPT_JOURNAL_SB;
 
 	if (ntohl(jsb->s_maxlen) < journal->j_maxlen)
 		journal->j_maxlen = ntohl(jsb->s_maxlen);
 	else if (ntohl(jsb->s_maxlen) > journal->j_maxlen)
-		return EXT2_ET_CORRUPT_SUPERBLOCK;
+		return EXT2_ET_CORRUPT_JOURNAL_SB;
 
 	journal->j_tail_sequence = ntohl(jsb->s_sequence);
 	journal->j_transaction_sequence = journal->j_tail_sequence;
