@@ -37,12 +37,12 @@ extern char *optarg;
 
 enum journal_location {JOURNAL_IS_INTERNAL, JOURNAL_IS_EXTERNAL};
 
-#define ANY_BLOCK ((blk_t) -1)
+#define ANY_BLOCK ((blk64_t) -1)
 
-int		dump_all, dump_contents, dump_descriptors;
-blk_t		block_to_dump, bitmap_to_dump, inode_block_to_dump;
-unsigned int	group_to_dump, inode_offset_to_dump;
-ext2_ino_t	inode_to_dump;
+static int		dump_all, dump_old, dump_contents, dump_descriptors;
+static blk64_t		block_to_dump, bitmap_to_dump, inode_block_to_dump;
+static unsigned int	group_to_dump, inode_offset_to_dump;
+static ext2_ino_t	inode_to_dump;
 
 struct journal_source
 {
@@ -94,6 +94,7 @@ void do_logdump(int argc, char **argv)
 	journal_source.fd = 0;
 	journal_source.file = 0;
 	dump_all = 0;
+	dump_old = 0;
 	dump_contents = 0;
 	dump_descriptors = 1;
 	block_to_dump = ANY_BLOCK;
@@ -102,7 +103,7 @@ void do_logdump(int argc, char **argv)
 	inode_to_dump = -1;
 
 	reset_getopt();
-	while ((c = getopt (argc, argv, "ab:ci:f:s")) != EOF) {
+	while ((c = getopt (argc, argv, "ab:ci:f:Os")) != EOF) {
 		switch (c) {
 		case 'a':
 			dump_all++;
@@ -126,6 +127,9 @@ void do_logdump(int argc, char **argv)
 			inode_spec = optarg;
 			dump_descriptors = 0;
 			break;
+		case 'O':
+			dump_old++;
+			break;
 		case 's':
 			use_sb++;
 			break;
@@ -162,7 +166,7 @@ void do_logdump(int argc, char **argv)
 			(group_offset / inodes_per_block);
 		inode_offset_to_dump = ((group_offset % inodes_per_block)
 					* sizeof(struct ext2_inode));
-		printf("Inode %u is at group %u, block %u, offset %u\n",
+		printf("Inode %u is at group %u, block %llu, offset %u\n",
 		       inode_to_dump, inode_group,
 		       inode_block_to_dump, inode_offset_to_dump);
 	}
@@ -267,48 +271,49 @@ errout:
 	return;
 
 print_usage:
-	fprintf(stderr, "%s: Usage: logdump [-acs] [-b<block>] [-i<filespec>]\n\t"
+	fprintf(stderr, "%s: Usage: logdump [-acsO] [-b<block>] [-i<filespec>]\n\t"
 		"[-f<journal_file>] [output_file]\n", argv[0]);
 }
 
 
 static int read_journal_block(const char *cmd, struct journal_source *source,
-			      off_t offset, char *buf, int size,
-			      unsigned int *got)
+			      off_t offset, char *buf, unsigned int size)
 {
 	int retval;
+	unsigned int got;
 
 	if (source->where == JOURNAL_IS_EXTERNAL) {
 		if (lseek(source->fd, offset, SEEK_SET) < 0) {
 			retval = errno;
-			com_err(cmd, retval, "while seeking in reading journal");
-			return retval;
+			goto seek_err;
 		}
 		retval = read(source->fd, buf, size);
-		if (retval >= 0) {
-			*got = retval;
-			retval = 0;
-		} else
+		if (retval < 0) {
 			retval = errno;
+			goto read_err;
+		}
+		got = retval;
+		retval = 0;
 	} else {
 		retval = ext2fs_file_lseek(source->file, offset,
 					   EXT2_SEEK_SET, NULL);
 		if (retval) {
+		seek_err:
 			com_err(cmd, retval, "while seeking in reading journal");
 			return retval;
 		}
-
-		retval = ext2fs_file_read(source->file, buf, size, got);
+		retval = ext2fs_file_read(source->file, buf, size, &got);
+		if (retval) {
+		read_err:
+			com_err(cmd, retval, "while reading journal");
+			return retval;
+		}
 	}
-
-	if (retval)
-		com_err(cmd, retval, "while reading journal");
-	else if (*got != (unsigned int) size) {
-		com_err(cmd, 0, "short read (read %d, expected %d) "
-			"while reading journal", *got, size);
+	if (got != size) {
+		com_err(cmd, 0, "short read (read %u, expected %u) "
+			"while reading journal", got, size);
 		retval = -1;
 	}
-
 	return retval;
 }
 
@@ -338,7 +343,6 @@ static void dump_journal(char *cmdname, FILE *out_file,
 	char			buf[8192];
 	journal_superblock_t	*jsb;
 	unsigned int		blocksize = 1024;
-	unsigned int		got;
 	int			retval;
 	__u32			magic, sequence, blocktype;
 	journal_header_t	*header;
@@ -347,8 +351,7 @@ static void dump_journal(char *cmdname, FILE *out_file,
 	unsigned int		blocknr = 0;
 
 	/* First, check to see if there's an ext2 superblock header */
-	retval = read_journal_block(cmdname, source, 0,
-				    buf, 2048, &got);
+	retval = read_journal_block(cmdname, source, 0, buf, 2048);
 	if (retval)
 		return;
 
@@ -361,7 +364,7 @@ static void dump_journal(char *cmdname, FILE *out_file,
 
 	if ((be32_to_cpu(jsb->s_header.h_magic) != JFS_MAGIC_NUMBER) &&
 	    (sb->s_magic == EXT2_SUPER_MAGIC) &&
-	    (sb->s_feature_incompat & EXT3_FEATURE_INCOMPAT_JOURNAL_DEV)) {
+	    ext2fs_has_feature_journal_dev(sb)) {
 		blocksize = EXT2_BLOCK_SIZE(sb);
 		blocknr = (blocksize == 1024) ? 2 : 1;
 		uuid_unparse(sb->s_uuid, jsb_buffer);
@@ -377,7 +380,7 @@ static void dump_journal(char *cmdname, FILE *out_file,
 	/* Next, read the journal superblock */
 
 	retval = read_journal_block(cmdname, source, blocknr*blocksize,
-				    jsb_buffer, 1024, &got);
+				    jsb_buffer, 1024);
 	if (retval)
 		return;
 
@@ -394,15 +397,19 @@ static void dump_journal(char *cmdname, FILE *out_file,
 	fprintf(out_file, "Journal starts at block %u, transaction %u\n",
 		blocknr, transaction);
 
-	if (!blocknr)
+	if (!blocknr) {
 		/* Empty journal, nothing to do. */
-		return;
+		if (!dump_old)
+			return;
+		else
+			blocknr = 1;
+	}
 
 	while (1) {
 		retval = read_journal_block(cmdname, source,
 					    blocknr*blocksize, buf,
-					    blocksize, &got);
-		if (retval || got != blocksize)
+					    blocksize);
+		if (retval)
 			return;
 
 		header = (journal_header_t *) buf;
@@ -421,7 +428,8 @@ static void dump_journal(char *cmdname, FILE *out_file,
 			fprintf (out_file, "Found sequence %u (not %u) at "
 				 "block %u: end of journal.\n",
 				 sequence, transaction, blocknr);
-			return;
+			if (!dump_old)
+				return;
 		}
 
 		if (dump_descriptors) {
@@ -460,6 +468,23 @@ static void dump_journal(char *cmdname, FILE *out_file,
 	}
 }
 
+static inline size_t journal_super_tag_bytes(journal_superblock_t *jsb)
+{
+	size_t sz;
+
+	if (JSB_HAS_INCOMPAT_FEATURE(jsb, JFS_FEATURE_INCOMPAT_CSUM_V3))
+		return sizeof(journal_block_tag3_t);
+
+	sz = sizeof(journal_block_tag_t);
+
+	if (JSB_HAS_INCOMPAT_FEATURE(jsb, JFS_FEATURE_INCOMPAT_CSUM_V2))
+		sz += sizeof(__u16);
+
+	if (JSB_HAS_INCOMPAT_FEATURE(jsb, JFS_FEATURE_INCOMPAT_64BIT))
+		return sz;
+
+	return sz - sizeof(__u32);
+}
 
 static void dump_descriptor_block(FILE *out_file,
 				  struct journal_source *source,
@@ -468,19 +493,21 @@ static void dump_descriptor_block(FILE *out_file,
 				  unsigned int *blockp, int blocksize,
 				  tid_t transaction)
 {
-	int			offset, tag_size = JBD_TAG_SIZE32;
+	int			offset, tag_size, csum_size = 0;
 	char			*tagp;
 	journal_block_tag_t	*tag;
 	unsigned int		blocknr;
 	__u32			tag_block;
 	__u32			tag_flags;
 
-	if (be32_to_cpu(jsb->s_feature_incompat) & JFS_FEATURE_INCOMPAT_64BIT)
-		tag_size = JBD_TAG_SIZE64;
-
+	tag_size = journal_super_tag_bytes(jsb);
 	offset = sizeof(journal_header_t);
 	blocknr = *blockp;
 
+	if (JSB_HAS_INCOMPAT_FEATURE(jsb, JFS_FEATURE_INCOMPAT_CSUM_V3) ||
+	    JSB_HAS_INCOMPAT_FEATURE(jsb, JFS_FEATURE_INCOMPAT_CSUM_V2))
+		csum_size = sizeof(struct journal_block_tail);
+
 	if (dump_all)
 		fprintf(out_file, "Dumping descriptor block, sequence %u, at "
 			"block %u:\n", transaction, blocknr);
@@ -497,11 +524,11 @@ static void dump_descriptor_block(FILE *out_file,
 
 		/* ... and if we have gone too far, then we've reached the
 		   end of this block. */
-		if (offset > blocksize)
+		if (offset > blocksize - csum_size)
 			break;
 
 		tag_block = be32_to_cpu(tag->t_blocknr);
-		tag_flags = be32_to_cpu(tag->t_flags);
+		tag_flags = be16_to_cpu(tag->t_flags);
 
 		if (!(tag_flags & JFS_FLAG_SAME_UUID))
 			offset += 16;
@@ -527,28 +554,37 @@ static void dump_revoke_block(FILE *out_file, char *buf,
 {
 	int			offset, max;
 	journal_revoke_header_t *header;
-	unsigned int		*entry, rblock;
+	unsigned long long	rblock;
+	int			tag_size = sizeof(__u32);
 
 	if (dump_all)
 		fprintf(out_file, "Dumping revoke block, sequence %u, at "
 			"block %u:\n", transaction, blocknr);
 
+	if (be32_to_cpu(jsb->s_feature_incompat) & JFS_FEATURE_INCOMPAT_64BIT)
+		tag_size = sizeof(__u64);
+
 	header = (journal_revoke_header_t *) buf;
 	offset = sizeof(journal_revoke_header_t);
 	max = be32_to_cpu(header->r_count);
 
 	while (offset < max) {
-		entry = (unsigned int *) (buf + offset);
-		rblock = be32_to_cpu(*entry);
+		if (tag_size == sizeof(__u32)) {
+			__u32 *entry = (__u32 *) (buf + offset);
+			rblock = be32_to_cpu(*entry);
+		} else {
+			__u64 *entry = (__u64 *) (buf + offset);
+			rblock = ext2fs_be64_to_cpu(*entry);
+		}
 		if (dump_all || rblock == block_to_dump) {
-			fprintf(out_file, "  Revoke FS block %u", rblock);
+			fprintf(out_file, "  Revoke FS block %llu", rblock);
 			if (dump_all)
 				fprintf(out_file, "\n");
 			else
 				fprintf(out_file," at block %u, sequence %u\n",
 					blocknr, transaction);
 		}
-		offset += 4;
+		offset += tag_size;
 	}
 }
 
@@ -576,7 +612,6 @@ static void dump_metadata_block(FILE *out_file, struct journal_source *source,
 				int blocksize,
 				tid_t transaction)
 {
-	unsigned int 	got;
 	int		retval;
 	char 		buf[8192];
 
@@ -612,7 +647,7 @@ static void dump_metadata_block(FILE *out_file, struct journal_source *source,
 
 	retval = read_journal_block("logdump", source,
 				    blocksize * log_blocknr,
-				    buf, blocksize, &got);
+				    buf, blocksize);
 	if (retval)
 		return;
 
@@ -624,7 +659,7 @@ static void dump_metadata_block(FILE *out_file, struct journal_source *source,
 		offset = ((block_to_dump - super->s_first_data_block) %
 			  super->s_blocks_per_group);
 
-		fprintf(out_file, "    (block bitmap for block %u: "
+		fprintf(out_file, "    (block bitmap for block %llu: "
 			"block is %s)\n",
 			block_to_dump,
 			ext2fs_test_bit(offset, buf) ? "SET" : "CLEAR");
